//  //
//  ###  ##### #   #               -= Confix Library =-                  //
// #   # #     ## ## ConfigFile.h - Configuration file                   //
// #     ####   ###                                                      //
// #     #      ###  Representation of a configuration file              //
// #   # #     ## ##                                                     //
//  ###  #     #   # R1             (C)2002 Markus Ewald -> License.txt  //
//  //
#ifndef CFX_CONFIGFILE_H
#define CFX_CONFIGFILE_H

#include "Confix/Config.h"
#include "Confix/ConfigSection.h"

#include <string>
#include <vector>

#ifdef CONFIX_NAMESPACE
  namespace CONFIX_NAMESPACE {
#endif CONFIX_NAMESPACE

//  //
//  CFX::CConfigFile                                                     //
//  //
/// A configuration file
/** Stores the contents of a configuration file using a simple object
    model to represent its sections and values.
*/
class CONFIX_API CConfigFile {
  public:
    /// Config section enumerator
    class CONFIX_API CSectionEnumerator;

    /// Constructor
    CConfigFile(void);
    /// Constructor
    CConfigFile(const std::string &sContents);
    /// Destructor
    ~CConfigFile(void);

  //
  // CConfigFile implementation
  //
  public:
    /// Get number of variables in file
    unsigned long getNumSections(void) const;

    /// Get variable
    CConfigSection &getSection(unsigned long lIndex);
    /// Get variable 
    CConfigSection &getSection(const std::string &sName);

    /// Add variable
    void addSection(const CConfigSection &Section);
    /// Remove variable
    void removeSection(unsigned long lIndex);
    /// Remove variable
    void removeSection(const std::string &sName);

    /// Get variable enumerator
    CSectionEnumerator getSections(void);

    /// Clear section
    void clearContents(void);
    /// Get configuration contents
    std::string getContents(void) const;
    /// Set configuration contents
    std::string::size_type setContents(const std::string &sContents);

  private:
    /// A map of configuration variables
    typedef std::vector<CConfigSection> SectionVector;

    SectionVector *m_pSections;                       ///< Sections in file
};

/** Enumerates over a list of config sections. An enumerator is a simple
    forward iterator which allows you to wander through a list of items
    without knowing the underlying data structure used for their storage.

    The enumerator starts in an empty position, meaning the first method
    you have to call is toNext(). This method returns true as long as it
    could advance to the next item. If there are no items available to be
    enumerated at all, toNext() will immediately return false.

    Usage is simple:
    @code
    CConfigFile::CSectionEnumerator SecEnum = MyCFGFile.getSections();
    while(SecEnum.toNext())
      cout << SecEnum.getSection().getName() << endl;
    @endcode
*/
class CONFIX_API CConfigFile::CSectionEnumerator {
  public:
    /// Constructor
    CSectionEnumerator(SectionVector &Sections);
    /// Copy constructor
    CSectionEnumerator(const CSectionEnumerator &SectionEnum);
    /// Destructor
    ~CSectionEnumerator(void);

  //
  // CSectionEnumerator implementation
  //
  public:
    /// Get current variable
    CConfigSection &getSection(void);

    /// Jump to next variable
    bool toNext(void);

  private:
    SectionVector           &m_Sections;            ///< Section vector
    SectionVector::iterator  m_SectionIt;           ///< Current variable
    SectionVector::iterator  m_SectionEnd;          ///< End variable
};

#ifdef CONFIX_NAMESPACE
  }; // CONFIX_NAMESPACE
#endif CONFIX_NAMESPACE

#endif CFX_CONFIGVARIABLE_H
